function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = _interopDefault(require('react'));
var d3Scale = require('d3-scale');
var d3Array = require('d3-array');
var d3Color = require('d3-color');
var d3Format = require('d3-format');
var d3Interpolate = require('d3-interpolate');
var d3Shape = require('d3-shape');
var d3Timer = require('d3-timer');
var d3Ease = require('d3-ease');

function _inheritsLoose(subClass, superClass) {
  subClass.prototype = Object.create(superClass.prototype);
  subClass.prototype.constructor = subClass;
  subClass.__proto__ = superClass;
}

var extractBidPrices = function extractBidPrices(data) {
  var set = new Set(data.map(function (d) {
    var marketDepth = d.marketDepth;

    if (marketDepth) {
      var buys = marketDepth.buys.map(function (b) {
        return +b.rate;
      });
      var sells = marketDepth.sells.map(function (b) {
        return +b.rate;
      });
      return buys.concat(sells, [+marketDepth.lastTradedPrice]);
    }

    return [];
  }).reduce(function (acc, val) {
    return acc.concat(val);
  }, []));
  return Array.from(set);
};
var extractBidVolumes = function extractBidVolumes(data) {
  var marketDepth = data.marketDepth;

  if (marketDepth) {
    var buys = marketDepth.buys.map(function (b) {
      return +b.qty;
    });
    var sells = marketDepth.sells.map(function (b) {
      return +b.qty;
    });
    return buys.concat(sells);
  } else return [];
};
var extractMaxVolume = function extractMaxVolume(data) {
  var vols = extractBidVolumes(data).sort(function (a, b) {
    return a - b;
  });
  if (vols.length > 0) return vols[vols.length - 1];else return 1;
};
var extractMaxTradedVolume = function extractMaxTradedVolume(data) {
  var vols = data.map(function (d) {
    if (d.marketDepth) return +d.marketDepth.lastTradedQty;else return 0;
  }).sort(function (a, b) {
    return a - b;
  });
  if (vols.length > 0) return vols[vols.length - 1];else return 1;
};
var zoomTimeFormat = function zoomTimeFormat(seconds, decimal) {
  if (!decimal) decimal = 2;

  if (seconds > 59) {
    if (seconds > 3599) {
      var hrs = seconds / 3600;
      return hrs.toFixed(decimal) + " hour" + (hrs > 1 ? 's' : '');
    } else {
      var mins = seconds / 60;
      return mins.toFixed(decimal) + " minute" + (mins > 1 ? 's' : '');
    }
  } else return seconds + " second" + (seconds > 1 ? 's' : '');
};

var d3 = Object.assign(Object.assign(Object.assign({}, d3Scale, d3Array, d3Color), d3Format, d3Interpolate, d3Shape), d3Ease, d3Timer);

var StockHeatmap = /*#__PURE__*/function (_React$Component) {
  _inheritsLoose(StockHeatmap, _React$Component);

  function StockHeatmap() {
    var _this;

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    _this = _React$Component.call.apply(_React$Component, [this].concat(args)) || this;
    _this.canvasRef = React.createRef();
    _this.drawingContext = null;
    _this.data = [];
    _this.windowedData = [];
    _this.windowLength = 40;
    _this.windowPosition = 0;
    _this.autoScroll = true;
    _this.defaults = {
      borderPadding: [5, 5, 0, 0],
      bidAskWidth: 100,
      axisYWidth: 50,
      axisXHeight: 50,
      buyColor: '#388e3c',
      textOnBuyColor: '#ffffff',
      sellColor: '#d32f2f',
      textOnSellColor: '#ffffff',
      textOnBackground: '#000000',
      textHighlightOnBackground: '#ff0000',
      tradeColor: '#7434eb',
      axisTickSize: 6,
      axisColor: '#000000',
      xAxisTextPadding: 6,
      yAxisTextPadding: 6,
      bidAskGraphPaddingLeft: 10,
      bidAskTransitionDuration: 500,
      volumeCircleMaxRadius: 10,
      runningRatioSeconds: 5,
      hmWidth: function hmWidth() {
        return _this.props.width - _this.defaults.borderPadding[1] - _this.defaults.borderPadding[3] - _this.defaults.bidAskWidth - _this.defaults.axisYWidth;
      },
      hmHeight: function hmHeight() {
        return _this.props.height - _this.defaults.borderPadding[0] - _this.defaults.borderPadding[2] - _this.defaults.axisXHeight;
      },
      clearColor: '#ffffff'
    };
    _this.isMouseDown = false;
    _this.mouseDownX = 0;

    _this.attachMouseListeners = function () {
      if (_this.canvasRef.current !== null) {
        _this.canvasRef.current.addEventListener('mousedown', _this.eventMouseDown);

        _this.canvasRef.current.addEventListener('mousemove', _this.eventMouseMove);

        _this.canvasRef.current.addEventListener('mouseup', _this.eventMouseUp);

        _this.canvasRef.current.addEventListener('wheel', _this.eventZoomWheel);

        _this.canvasRef.current.addEventListener('keydown', _this.eventKeyDown);
      }
    };

    _this.detachMouseListeners = function () {
      if (_this.canvasRef.current !== null) {
        _this.canvasRef.current.removeEventListener('mousedown', _this.eventMouseDown);

        _this.canvasRef.current.removeEventListener('mousemove', _this.eventMouseMove);

        _this.canvasRef.current.removeEventListener('mouseup', _this.eventMouseUp);

        _this.canvasRef.current.removeEventListener('wheel', _this.eventZoomWheel);

        _this.canvasRef.current.removeEventListener('keydown', _this.eventKeyDown);
      }
    };

    _this.eventMouseDown = function (e) {
      _this.isMouseDown = true;
      _this.mouseDownX = e.x;
    };

    _this.eventMouseMove = function (e) {
      if (_this.isMouseDown) {
        var dragLength = e.x - _this.mouseDownX;
        var moveDataPointsCount = Math.floor(Math.abs(dragLength) / _this.xScale.bandwidth());
        if (moveDataPointsCount > 0) _this.mouseDownX = e.x;

        _this.moveDataWindow(_this.windowPosition + moveDataPointsCount * (dragLength >= 0 ? -1 : 1));
      }
    };

    _this.eventMouseUp = function (e) {
      _this.isMouseDown = false;
      _this.mouseDownX = 0;
    };

    _this.eventZoomWheel = function (e) {
      var direction = e.deltaY < 0 ? 'zoom-in' : 'zoom-out';
      var l = 0,
          l2 = 0;

      switch (direction) {
        case 'zoom-in':
          l = Math.max(_this.windowLength - 1, 3);
          break;

        case 'zoom-out':
          l = Math.min(_this.windowLength + 1, _this.data.length - 1);
          break;
      }

      l2 = _this.windowLength - l;
      _this.windowLength = l;

      _this.moveDataWindow(_this.windowPosition + l2);
    };

    _this.eventKeyDown = function (e) {
      e.preventDefault();
      console.log('key event', e.isComposing, e.key, e.ctrlKey);

      switch (e.key) {
        case 'ArrowLeft':
          _this.moveDataWindow(_this.windowPosition - (e.ctrlKey ? 10 : 1));

          break;

        case 'ArrowRight':
          _this.moveDataWindow(_this.windowPosition + (e.ctrlKey ? 10 : 1));

          break;
      }
    };

    _this.xScale = null;
    _this.bidAskScale = null;
    _this.yScale = null;
    _this.yDomainValues = null;
    _this.bidAskAnimTimer = null;
    _this.bidAskBarAnimConfig = {};

    _this.updateHeatmapDimensions = function () {
      var _this$props = _this.props,
          width = _this$props.width,
          height = _this$props.height;

      if (width > 0 && height > 0 && _this.windowedData.length > 0) {
        _this.xScale = d3.scaleBand().range([0, _this.defaults.hmWidth()]).domain(_this.windowedData.map(function (d) {
          return d.ts;
        }));
        _this.yDomainValues = extractBidPrices(_this.windowedData).sort(function (a, b) {
          return a - b;
        });
        _this.yScale = d3.scaleBand().range([_this.defaults.hmHeight(), 0]).domain(_this.yDomainValues);
        _this.bidAskScale = d3.scaleLinear().range([0, _this.defaults.bidAskWidth]).domain([0, d3.max(extractBidVolumes(_this.windowedData[_this.windowedData.length - 1]))]);
      }
    };

    _this.updateHeatmap = function () {
      if (_this.drawingContext !== null) {
        _this.updateHeatmapDimensions();

        _this.clearCanvas(_this.defaults.borderPadding[3], _this.defaults.borderPadding[0], _this.defaults.hmWidth(), _this.defaults.hmHeight(), _this.defaults.clearColor);

        _this.drawMainGraph();

        _this.drawXAxis();

        _this.drawYAxisAndBidAskGraph();

        _this.drawBuy2SellRatio();
      }
    };

    _this.drawBuy2SellRatio = function () {
      if (_this.windowedData.length > 0) {
        var d = _this.windowedData[_this.windowedData.length - 1];

        var x = _this.defaults.borderPadding[3] + _this.defaults.hmWidth() + _this.defaults.axisTickSize;

        var y = _this.defaults.borderPadding[0] + _this.defaults.hmHeight() + _this.defaults.axisTickSize;

        var w = _this.props.width - x;
        var h = _this.props.height - y;

        _this.clearCanvas(x, y, w, h, _this.defaults.clearColor);

        var textHeight = (h - 10) / 2;

        _this.drawingContext.save();

        _this.drawingContext.textAlign = 'center';
        _this.drawingContext.textBaseline = 'middle';
        _this.drawingContext.font = "bold " + textHeight + "px sans-serif";

        _this.drawingContext.fillText((d.marketDepth.buyOrderVolume / d.marketDepth.sellOrderVolume).toFixed(2), x + w * 3 / 4, y + textHeight / 2);

        if (_this.windowedData.length >= _this.defaults.runningRatioSeconds) {
          var sellT20RunningSum = 0;
          var buyT20RunningSum = 0;

          for (var i = _this.windowedData.length - 1; i >= _this.windowedData.length - _this.defaults.runningRatioSeconds; i--) {
            sellT20RunningSum += (_this.windowedData[i].marketDepth.sells || []).reduce(function (vol, s) {
              return vol + s.qty;
            }, 0);
            buyT20RunningSum += (_this.windowedData[i].marketDepth.buys || []).reduce(function (vol, s) {
              return vol + s.qty;
            }, 0);
          }

          var newBSTPFactor = buyT20RunningSum / sellT20RunningSum;

          _this.drawingContext.fillText(newBSTPFactor.toFixed(2), x + w / 4, y + textHeight * 0.5);
        }

        _this.drawingContext.font = "bold " + 13 + "px sans-serif";
        _this.drawingContext.textBaseline = 'bottom';

        _this.drawingContext.fillText('Buy/Sell', x + w / 2, y + textHeight * 2 + 5);

        _this.drawingContext.restore();
      }
    };

    _this.drawXAxis = function () {
      var _this$xScale;

      _this.clearCanvas(_this.defaults.borderPadding[3], _this.defaults.borderPadding[0] + _this.defaults.hmHeight(), _this.defaults.hmWidth(), _this.defaults.axisXHeight, _this.defaults.clearColor);

      _this.drawingContext.save();

      _this.drawingContext.beginPath();

      _this.drawingContext.translate(_this.defaults.borderPadding[3], _this.defaults.borderPadding[0] + _this.defaults.hmHeight());

      _this.drawingContext.moveTo(0, 0);

      _this.drawingContext.lineTo(_this.defaults.hmWidth(), 0);

      _this.drawingContext.textAlign = 'center';
      _this.drawingContext.textBaseline = 'top';
      var assumedTextWidth = _this.drawingContext.measureText('77:77:77').width + 20;
      var bandInterval = parseInt(assumedTextWidth / (((_this$xScale = _this.xScale) === null || _this$xScale === void 0 ? void 0 : _this$xScale.bandwidth()) || 1)) || 1;

      _this.windowedData.map(function (d, i) {
        if (i % bandInterval === 0) {
          var x = _this.xScale(d.ts);

          _this.drawingContext.moveTo(x, 0);

          _this.drawingContext.lineTo(x, _this.defaults.axisTickSize);

          _this.drawingContext.fillText(d.ts, x, _this.defaults.axisTickSize + _this.defaults.xAxisTextPadding);
        }
      });

      _this.drawingContext.textAlign = 'left';
      _this.drawingContext.font = '12px Arial';

      _this.drawingContext.fillText("Zoom Level:  " + zoomTimeFormat(_this.windowLength), 20, _this.defaults.axisTickSize + _this.defaults.xAxisTextPadding + 20);

      var w = _this.drawingContext.measureText("Zoom Level:  " + zoomTimeFormat(_this.windowLength)).width;

      var maxVolumeInWindowData = extractMaxTradedVolume(_this.windowedData);

      _this.drawingContext.fillText("Max Volume in " + zoomTimeFormat(_this.windowLength, 1) + ":  ", 20 + w + 20, _this.defaults.axisTickSize + _this.defaults.xAxisTextPadding + 20);

      _this.drawingContext.fillStyle = _this.defaults.textHighlightOnBackground;
      w += _this.drawingContext.measureText("Max Volume in " + zoomTimeFormat(_this.windowLength, 1) + ":  ").width;
      _this.drawingContext.font = 'bold 12px Arial';

      _this.drawingContext.fillText("" + maxVolumeInWindowData, 20 + w + 20, _this.defaults.axisTickSize + _this.defaults.xAxisTextPadding + 20);

      w += _this.drawingContext.measureText("" + maxVolumeInWindowData).width;

      if (_this.windowedData.length > 0) {
        _this.drawingContext.fillStyle = _this.defaults.textOnBackground;

        _this.drawingContext.fillText("LTP:  " + _this.windowedData[_this.windowedData.length - 1].marketDepth.lastTradedPrice + "     LTQ:  " + _this.windowedData[_this.windowedData.length - 1].marketDepth.lastTradedQty, 20 + w + 40, _this.defaults.axisTickSize + _this.defaults.xAxisTextPadding + 20);
      }

      _this.drawingContext.fillStyle = _this.defaults.textOnBackground;
      _this.drawingContext.lineWidth = 1.2;
      _this.drawingContext.strokeStyle = _this.defaults.axisColor;

      _this.drawingContext.stroke();

      _this.drawingContext.restore();
    };

    _this.drawYAxisAndBidAskGraph = function () {
      if (_this.yDomainValues !== null) {
        _this.clearCanvas(_this.defaults.borderPadding[3] + _this.defaults.hmWidth(), _this.defaults.borderPadding[0], _this.defaults.axisYWidth, _this.defaults.hmHeight() + _this.defaults.axisTickSize, _this.defaults.clearColor);

        _this.drawingContext.save();

        _this.drawingContext.beginPath();

        _this.drawingContext.translate(_this.defaults.borderPadding[3] + _this.defaults.hmWidth(), _this.defaults.borderPadding[0]);

        _this.drawingContext.moveTo(0, 0);

        _this.drawingContext.lineTo(0, _this.defaults.hmHeight() + _this.defaults.axisTickSize);

        _this.drawingContext.textAlign = 'start';
        _this.drawingContext.textBaseline = 'top';
        var maxTextWidth = 0;

        _this.yDomainValues.map(function (d) {
          var y = _this.yScale(d);

          _this.drawingContext.moveTo(0, y);

          _this.drawingContext.lineTo(_this.defaults.axisTickSize, y);

          _this.drawingContext.fillText(d.toFixed(2), _this.defaults.axisTickSize + _this.defaults.yAxisTextPadding, y + 2, _this.defaults.axisYWidth - _this.defaults.axisTickSize + _this.defaults.yAxisTextPadding);

          var tw = _this.drawingContext.measureText(d.toFixed(2)).width;

          maxTextWidth = maxTextWidth >= tw ? maxTextWidth : tw;
        });

        _this.drawingContext.lineWidth = 1.2;
        _this.drawingContext.strokeStyle = _this.defaults.axisColor;

        _this.drawingContext.stroke();

        _this.drawingContext.restore();

        var x = _this.defaults.borderPadding[3] + _this.defaults.hmWidth() + maxTextWidth + _this.defaults.axisTickSize + _this.defaults.yAxisTextPadding + _this.defaults.bidAskGraphPaddingLeft;

        var y = _this.defaults.borderPadding[0];

        _this.drawBidAskGraph(x, y);
      }
    };

    _this.drawBidAskGraph = function (x, y) {
      if (_this.windowedData.length > 0) {
        if (_this.bidAskAnimTimer !== null) {
          _this.bidAskAnimTimer.stop();

          _this.bidAskAnimTimer = null;
        }

        _this.bidAskAnimTimer = d3.timer(function (elapsed) {
          var t = Math.min(1, d3.easeCubic(elapsed / _this.defaults.bidAskTransitionDuration));

          _this.clearCanvas(x, y, _this.defaults.bidAskWidth, _this.defaults.hmHeight() + _this.defaults.axisTickSize, _this.defaults.clearColor);

          var h = _this.yScale.bandwidth() - 2;
          var d = _this.windowedData[_this.windowedData.length - 1];
          var maxBidAskVol = extractMaxVolume(d);

          _this.drawingContext.save();

          _this.drawingContext.translate(x, y);

          _this.drawingContext.lineWidth = 0;
          _this.drawingContext.textBaseline = 'middle';

          var drawBars = function drawBars(arr, color, textColor) {
            arr.map(function (v) {
              _this.drawingContext.fillStyle = color;
              var l = _this.defaults.bidAskWidth * (+v.qty / maxBidAskVol);
              _this.bidAskBarAnimConfig[v.rate] = d3.interpolateNumber(_this.bidAskBarAnimConfig[v.rate] || 0, l)(t);

              _this.drawingContext.fillRect(0, _this.yScale(v.rate), _this.bidAskBarAnimConfig[v.rate], h);

              var tw = _this.drawingContext.measureText(v.qty).width;

              if (_this.defaults.bidAskWidth - _this.bidAskBarAnimConfig[v.rate] - 2 >= tw) {
                _this.drawingContext.textAlign = 'start';
                _this.drawingContext.fillStyle = _this.defaults.textOnBackground;

                _this.drawingContext.fillText(v.qty, _this.bidAskBarAnimConfig[v.rate] + 2, _this.yScale(v.rate) + h / 2 + 1);
              } else {
                _this.drawingContext.textAlign = 'end';
                _this.drawingContext.fillStyle = textColor;

                _this.drawingContext.fillText(v.qty, _this.bidAskBarAnimConfig[v.rate] - 2, _this.yScale(v.rate) + h / 2 + 1);
              }
            });
          };

          drawBars(d.marketDepth.buys, _this.defaults.buyColor, _this.defaults.textOnBuyColor);
          drawBars(d.marketDepth.sells, _this.defaults.sellColor, _this.defaults.textOnSellColor);

          _this.drawingContext.restore();

          if (t === 1) _this.bidAskAnimTimer.stop();
        });
      }
    };

    _this.drawMainGraph = function () {
      _this.drawingContext.save();

      if (_this.xScale && _this.yScale && _this.bidAskScale && _this.drawingContext !== null) {
        var maxTradedVolume = extractMaxTradedVolume(_this.windowedData);
        var xh2 = _this.xScale.bandwidth() * 0.5;
        var yh2 = _this.yScale.bandwidth() * 0.5;

        _this.drawingContext.translate(_this.defaults.borderPadding[3], _this.defaults.borderPadding[0]);

        _this.windowedData.map(function (d) {
          var marketDepth = d.marketDepth;
          var ts = d.ts;
          var maxBidAskVol = extractMaxVolume(d);

          if (marketDepth.buys && marketDepth.buys.length > 0) {
            var _color = d3.color(_this.defaults.buyColor).rgb();

            marketDepth.buys.map(function (buy) {
              _color.opacity = buy.qty / maxBidAskVol;
              _this.drawingContext.fillStyle = _color.toString();

              _this.drawingContext.fillRect(_this.xScale(ts), _this.yScale(buy.rate), _this.xScale.bandwidth(), _this.yScale.bandwidth());
            });
          }

          if (marketDepth.sells && marketDepth.sells.length > 0) {
            var _color2 = d3.color(_this.defaults.sellColor).rgb();

            marketDepth.sells.map(function (sell) {
              _color2.opacity = sell.qty / maxBidAskVol;
              _this.drawingContext.fillStyle = _color2.toString();

              _this.drawingContext.fillRect(_this.xScale(ts), _this.yScale(sell.rate), _this.xScale.bandwidth(), _this.yScale.bandwidth());
            });
          }

          var color = d3.color(_this.defaults.tradeColor).rgb();
          color.opacity = 1;
          _this.drawingContext.lineWidth = 1;
          _this.drawingContext.fillStyle = color.toString();
          var r = _this.defaults.volumeCircleMaxRadius * (+marketDepth.lastTradedQty / maxTradedVolume);

          _this.drawingContext.beginPath();

          _this.drawingContext.arc(_this.xScale(ts), _this.yScale(+marketDepth.lastTradedPrice), r, 0, 2 * Math.PI);

          _this.drawingContext.fill();
        });

        _this.drawingContext.beginPath();

        d3.line().x(function (d) {
          return _this.xScale(d.ts);
        }).y(function (d) {
          return _this.yScale(+d.marketDepth.lastTradedPrice);
        }).curve(d3.curveLinear).context(_this.drawingContext)(_this.windowedData);
        _this.drawingContext.lineWidth = 1;
        _this.drawingContext.strokeStyle = _this.defaults.tradeColor;

        _this.drawingContext.stroke();
      }

      _this.drawingContext.restore();
    };

    _this.clearCanvas = function (x, y, w, h, color) {
      if (_this.drawingContext !== null) {
        _this.drawingContext.save();

        _this.drawingContext.fillStyle = color || _this.defaults.clearColor;

        _this.drawingContext.fillRect(x, y, w, h);

        _this.drawingContext.restore();
      }
    };

    _this.setData = function (data) {
      if (data && data.length > 0) {
        _this.data = data;

        _this.updateWindowedData();
      }
    };

    _this.addData = function (data) {
      if (typeof data === 'object') {
        _this.data.push(data);

        _this.updateWindowedData();
      }
    };

    _this.updateWindowedData = function () {
      _this.moveDataWindow(_this.data.length - _this.windowLength - 1);
    };

    _this.moveDataWindow = function (position) {
      if (position !== _this.windowPosition && position > -1 && position < _this.data.length - _this.windowLength) {
        _this.windowedData = _this.data.slice(position, position + _this.windowLength);
        _this.windowPosition = position;

        if (_this.windowPosition === _this.data.length - _this.windowLength - 1) {
          _this.autoScroll = true;
        }

        _this.updateHeatmap();
      }
    };

    _this.setZoomLevel = function (zoom) {
      var l = Math.min(Math.max(zoom, 3), _this.data.length - 1);
      var l2 = _this.windowLength - l;
      _this.windowLength = l;

      _this.moveDataWindow(_this.windowPosition + l2);
    };

    return _this;
  }

  var _proto = StockHeatmap.prototype;

  _proto.shouldComponentUpdate = function shouldComponentUpdate(nextProps, nextState) {
    var shouldUpdate = this.props.width !== nextProps.width || this.props.height !== nextProps.height;

    if (shouldUpdate) {
      this.detachMouseListeners();
    }

    return shouldUpdate;
  };

  _proto.componentDidMount = function componentDidMount() {
    if (this.canvasRef.current !== null) {
      this.drawingContext = this.canvasRef.current.getContext('2d');
      this.updateHeatmap();
      this.attachMouseListeners();
    }
  };

  _proto.componentDidUpdate = function componentDidUpdate() {
    if (this.canvasRef.current !== null) {
      this.drawingContext = this.canvasRef.current.getContext('2d');
      this.updateHeatmap();
      this.attachMouseListeners();
    }
  };

  _proto.componentWillUnmount = function componentWillUnmount() {
    this.detachMouseListeners();
  };

  _proto.render = function render() {
    var _this$props2 = this.props,
        width = _this$props2.width,
        height = _this$props2.height;
    return /*#__PURE__*/React.createElement("canvas", {
      ref: this.canvasRef,
      width: width || 300,
      height: height || 150,
      tabIndex: 1,
      style: {
        display: 'block',
        width: '100%',
        height: '100%',
        cursor: 'crosshair'
      }
    });
  };

  return StockHeatmap;
}(React.Component);

exports.d3 = d3;
exports.default = StockHeatmap;
//# sourceMappingURL=index.js.map
